//Starpost Reward script - KHBlast Edition
//Original by WellDoneSnake, modified by Latius
//Remake of starpost reward system from their misc. pack release in 2.1.

local function HUDIconSetup(player, timer, patch, trans)
	if player.mo and player.mo.valid
		player.icontimer = timer
		player.icongift = patch
		player.icontransp = trans
	end
end

//Reward Setup
local ringset = {1, 5, 10, 15, 20, 25, 30, 50}
local xpset = {10, 100, 100, 100, 200, 200, 200, 400, 400, 800, 800, 1000}
local rpset = {2, 5, 10, 15, 20, 30, 50}
local itemset = {1, 1, 1, 2, 2, 4, 6, 6, 9, 11}
local shieldset = {SH_WHIRLWIND, SH_ARMAGEDDON, SH_ELEMENTAL, SH_FORCE|1, SH_BUBBLEWRAP, SH_ATTRACT, SH_FLAMEAURA, SH_THUNDERCOIN}
local shieldiconset = {"TVWWC0", "TVARC0", "TVELC0", "TVFOC0", "TVBBC0", "TVATC0", "TVFLC0", "TVZPC0"}

local DRIVEBONUS = 0
local XPBONUS = 1
local RPBONUS = 2
local ITEMBONUS = 3
local SHIELDBONUS = 4
local SNEAKERBONUS = 5
local INVINCIBILITYBONUS = 6
local AUTOLIFEBONUS = 7
local TOTALSETS = 8
local TOTALWEAKSETS = 5

local function GiveRandomReward(player)
	local set -- 0 = rings, 1 = shields, 2 = sneakers, 3 = invincibility, 4 = lives
	if player.rings >= 50 then set = P_RandomKey(TOTALSETS) else set = P_RandomKey(TOTALWEAKSETS) end

	if set == SHIELDBONUS and (player.powers[pw_shield] != SH_NONE) then -- Don't reward the player with another shield, instead give them rings or the invincibility power-up
		if (player.rings >= 50) then
			local chance = P_RandomKey(TOTALWEAKSETS)
			if chance == SHIELDBONUS then set = INVINCIBILITYBONUS else set = chance end
		else
			set = P_RandomKey(TOTALWEAKSETS - 1)
		end
	end
	
	if set == RPBONUS then
		if not player.kh then set = DRIVEBONUS
		elseif (player.kh.maxRP == nil) or (player.kh.maxRP < 1) or (player.kh.rp == player.kh.maxRP) then 
			set = P_RandomKey(3)
			if set == RPBONUS then set = $ + 1 end
		end
	end
	
	if set == ITEMBONUS and (not player.kh) then
		set = DRIVEBONUS
	end

	if set == DRIVEBONUS
		P_GivePlayerRings(player, ringset[P_RandomKey(#ringset) + 1])
		S_StartSound(player.mo, sfx_itemup, player)
		HUDIconSetup(player, 3*TICRATE, "TVRIC0", 0)
	elseif set == XPBONUS
		local expbonus = xpset[P_RandomKey(#xpset) + 1]
		P_AddPlayerScore(player, expbonus)
		S_StartSound(player.mo, sfx_itemup, player)
		local icon = "SCOR"
		if expbonus == 10 then icon = $ .. "L0"
		elseif expbonus == 100 then icon = $ .. "A0"
		elseif expbonus == 200 then icon = $ .. "B0"
		elseif expbonus == 400 then icon = $ .. "I0"
		elseif expbonus == 800 then icon = $ .. "J0"
		else icon = $ .. "D0" end
		HUDIconSetup(player, 3*TICRATE, icon, 0)
	elseif set == RPBONUS
		player.kh.rpcharge = $ + rpset[P_RandomKey(min(#rpset, (player.kh.maxRP/2)+1))+ 1]
		if player.kh.rpcharge >= 5 then
			local rpbonus = player.kh.rpcharge / 5
			player.kh.rp = min($ + rpbonus, player.kh.maxRP)
			if player.kh.rp == player.kh.maxRP then player.kh.rpcharge = 0 else player.kh.rpcharge = $ % 5 end
		end
		S_StartSound(player.mo, sfx_itemup, player)
		HUDIconSetup(player, 3*TICRATE, "TV1KC0", 0)
	elseif set == ITEMBONUS
		local drop = itemset[P_RandomKey(#itemset) + 1]
		player.kh.items[drop] = $ + 1
		player.kh.itempickup = drop
		player.kh.itempickuptimer = TICRATE * 6
		player.kh.itempickuptext = nil
		S_StartSound(nil, sfx_itmget, player)
		local numitems = #player.kh.itemlist
		if numitems > 0 then
			for i = 1, numitems do //Is it in the table?
				if player.kh.itemlist[i] == drop then return false end //Yes, so we're done here.
			end
		end
		//No, so add it and sort the table.
		table.insert(player.kh.itemlist, drop)
		local itemoption
		if player.kh.menuMode == 1 then
			itemoption = player.kh.itemlist[p.kh.cOption]
		end
		table.sort(player.kh.itemlist)
		if itemoption ~= nil and itemoption ~= player.kh.itemlist[player.kh.cOption] then
			player.kh.cOption = $ + 1
		end
	elseif set == SHIELDBONUS
		local random
		if player.mo.eflags & (MFE_TOUCHWATER|MFE_UNDERWATER)
			random = P_RandomRange(1, #shieldset - 3)
		else
			random = P_RandomRange(1, #shieldset)
		end
		player.powers[pw_shield] = shieldset[random]
		P_SpawnShieldOrb(player)
		S_StartSound(player.mo, sfx_shield, player)
		HUDIconSetup(player, 3*TICRATE, shieldiconset[random],0)
	elseif set == SNEAKERBONUS
		player.powers[pw_sneakers] = sneakertics + 1 -- Stolen from the source code :]
		P_PlayJingle(player, JT_SHOES)
		HUDIconSetup(player, 3*TICRATE, "TVSSC0", 0)
	elseif set == INVINCIBILITYBONUS
		player.powers[pw_invulnerability] = invulntics + 1 -- Also stolen from the source code :P
		P_PlayJingle(player, JT_INV)
		HUDIconSetup(player, 3*TICRATE, "TVIVC0", 0)
	elseif set == AUTOLIFEBONUS
		P_GivePlayerLives(player, 1)
		P_PlayLivesJingle(player)
		HUDIconSetup(player, 3*TICRATE, "TV1UC0", 0)
	end
end

//Handle HUD timer & transparency in the player's thinker
addHook("ThinkFrame", do
	for player in players.iterate
		if player.icontimer != nil and player.icontimer > 0
			player.icontimer = $ - 1
			if player.icontimer < TICRATE 
				player.icontransp = $ + 1
				if player.icontransp > 9
					player.icontimer = 0
				end
			end
		end
	end
end)

//Starpost Interaction
local function ItemReward(post, mo)
	local player = mo.player
	if player.bot return end //Don't give bonuses to bots.
	//Activate starpost once
	if player.starpostnum < post.health -- Learnt this from FuriousFox's "BonusStage" addon on MB (2.1 archives)
		GiveRandomReward(player)
	end
end
addHook("TouchSpecial", ItemReward, MT_STARPOST)

//HUD Drawing Power-Up Icons
local function hud_icon(v, stplyr)
	if stplyr.icontimer != nil and stplyr.icontimer > 0
		local icontransp = stplyr.icontransp<<V_ALPHASHIFT
		local iconpatch = (v.cachePatch(stplyr.icongift))
		v.draw(160, 170, iconpatch, icontransp|V_SNAPTOBOTTOM)
	end
end
hud.add(hud_icon)